@@ -262,6 +262,10 @@ def execute(*cmd, **kwargs):
     :param prlimit:         Set resource limits on the child process. See
                             below for a detailed description.
     :type prlimit:          :class:`ProcessLimits`
+    :param python_exec:     The python executable to use for enforcing
+                            prlimits. If this is not set it will default to use
+                            sys.executable.
+    :type python_exec:      string
     :returns:               (stdout, stderr) from process execution
     :raises:                :class:`UnknownArgumentError` on
                             receiving unknown arguments
@@ -314,6 +318,7 @@ def execute(*cmd, **kwargs):
     on_completion = kwargs.pop('on_completion', None)
     preexec_fn = kwargs.pop('preexec_fn', None)
     prlimit = kwargs.pop('prlimit', None)
+    python_exec = kwargs.pop('python_exec', sys.executable)
 
     if isinstance(check_exit_code, bool):
         ignore_exit_code = not check_exit_code
@@ -350,7 +355,7 @@ def execute(*cmd, **kwargs):
                     _('Process resource limits are ignored as '
                       'this feature is not supported on Windows.'))
         else:
-            args = [sys.executable, '-m', 'oslo_concurrency.prlimit']
+            args = [python_exec, '-m', 'oslo_concurrency.prlimit']
             args.extend(prlimit.prlimit_args())
             args.append('--')
             args.extend(cmd)
@@ -381,7 +386,7 @@ def execute(*cmd, **kwargs):
                                    stderr=_PIPE,
                                    close_fds=close_fds,
                                    preexec_fn=on_preexec_fn,
-                                   shell=shell,
+                                   shell=shell,  # nosec:B604
                                    cwd=cwd,
                                    env=env_variables)
 
@@ -498,9 +503,21 @@ def trycmd(*args, **kwargs):
 
 def ssh_execute(ssh, cmd, process_input=None,
                 addl_env=None, check_exit_code=True,
-                binary=False, timeout=None):
+                binary=False, timeout=None,
+                sanitize_stdout=True):
     """Run a command through SSH.
 
+    :param ssh:             An SSH Connection object.
+    :param cmd:             The command string to run.
+    :param check_exit_code: If an exception should be raised for non-zero
+                            exit.
+    :param timeout:         Max time in secs to wait for command execution.
+    :param sanitize_stdout: Defaults to True. If set to True, stdout is
+                            sanitized i.e. any sensitive information like
+                            password in command output will be masked.
+    :returns:               (stdout, stderr) from command execution through
+                            SSH.
+
     .. versionchanged:: 1.9
        Added *binary* optional parameter.
     """
@@ -532,13 +549,21 @@ def ssh_execute(ssh, cmd, process_input=None,
         # mask_password() requires Unicode on Python 3
         stdout = os.fsdecode(stdout)
         stderr = os.fsdecode(stderr)
-    stdout = strutils.mask_password(stdout)
+
+    if sanitize_stdout:
+        stdout = strutils.mask_password(stdout)
+
     stderr = strutils.mask_password(stderr)
 
     # exit_status == -1 if no exit code was returned
     if exit_status != -1:
         LOG.debug('Result was %s' % exit_status)
         if check_exit_code and exit_status != 0:
+            # In case of errors in command run, due to poor implementation of
+            # command executable program, there might be chance that it leaks
+            # sensitive information like password to stdout. In such cases
+            # stdout needs to be sanitized even though sanitize_stdout=False.
+            stdout = strutils.mask_password(stdout)
             raise ProcessExecutionError(exit_code=exit_status,
                                         stdout=stdout,
                                         stderr=stderr,
@@ -550,9 +575,9 @@ def ssh_execute(ssh, cmd, process_input=None,
             # to decode it, or an Unicode string otherwise. Encode to the
             # default encoding (ASCII) because mask_password() decodes from
             # the same encoding.
-            if isinstance(stdout, unicode):
+            if isinstance(stdout, six.text_type):
                 stdout = stdout.encode()
-            if isinstance(stderr, unicode):
+            if isinstance(stderr, six.text_type):
                 stderr = stderr.encode()
         else:
             # fsencode() is the reverse operation of fsdecode()
